var app = angular.module('siteBlocker', []);

app.controller('MainController', [
    '$scope',
    '$filter',
    '$interval',
    '$timeout',
    'HelperService',
    '$http',
    function($scope, $filter, $interval, $timeout, HelperService, $http) {
        init();
        function init() {
            var manifest = chrome.runtime.getManifest();
            $scope.appVersion = manifest.version;
            $scope.siteList = [];

            // We have ng-if so need to use object reference
            $scope.model = {
                accessPassword: '',
                errorSite: false,
                newSite: '',
                // When click on site to unblock
                sitetoUnblock: '',
                sitetoIsUnblocked: false,
                showPassInput: HelperService.getPass(),
                displayContent: !HelperService.getPass(),
                sortReverse: false,
                newBlockKey: '',
                timeRange: [],
                now: '',
                minTime: 0,
                maxTime: 24,
                hour_config_extra: '',
                redirect_url: localStorage.getItem('redirect_url'),
                currentPage: 'dashboard',
                exportDataModel: '',
                loadDataListModel: '',
                loadDataResult: [],
                loadDataArray: [],
                userPassword: ''
            };

            $scope.days = [];
            $scope.dayList = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];

            if (!$scope.model.displayContent) {
                $('#accessDashboardPassword').modal();
                focus('#access_user_password', 1000);
            }

            $scope.checkbox = {};
            $scope.not = {
                list: [],
                show: false,
                read: [],
                unRead: 0,
            };

            $scope.checkbox.optionCloseTabs = parseInt(HelperService.getKey('close_option'));

            readStorageConfig();

            var times = [];
            for (var i = 0; i < 24; i++) {
                for (var j = 0; j < 60; j += 10) {
                    var t = i < 10 ? '0' + i : i;
                    var tt = j < 10 ? '0' + j : j;
                    times.push(t + ':' + tt);
                }
            }

            $scope.model.timeRange = times;
            tick();

            function tick() {
                var date = new Date();
                $scope.model.now = $filter('date')(date, 'HH:mm:ss');
            }

            $interval(tick, 1000);

            var hour_config = HelperService.getKey('hour_config', true, {});

            $scope.model.hour_config_extra = HelperService.getExtraHour();

            $scope.addExtraTime = function() {
                $scope.model.hour_config_extra.push({ start: '18:00', end: '21:00' });
            };

            $scope.removeHourItem = function(index) {
                $scope.model.hour_config_extra.splice(index, 1);
            };

            var loactionHash = document.location.hash.substring(1);

            chrome.runtime.sendMessage(
                {
                    method: 'getStatus',
                },
                function(response) {
                    $scope.siteList = response.status ? JSON.parse(response.status) : [];
                    if ($scope.siteList.indexOf(loactionHash) != -1) {
                        $scope.model.sitetoUnblock = loactionHash;
                    }
                    $scope.$apply();
                }
            );

            var oneDay = 86400000;
            var lastUpdate = localStorage.getItem('lastUpdate') || 0;
            var now = +new Date();
            if (now - lastUpdate > oneDay) {
                loadNotifications();
            } else {
                $scope.not.list = HelperService.getKey('notifications', true, []) || [];
                $scope.not = HelperService.getReadMessages($scope.not);
            }

        }

        // Check enterd password when user go to dashboard
        $scope.checkPassword = function() {
            if (!$scope.model.accessPassword) {
                focus('#access_user_password', 10);
                return;
            }
            if (HelperService.isPasswordValid($scope.model.accessPassword)) {
                $scope.model.accessPassword = '';
                $scope.model.displayContent = true;

                // If need to unblock site
                if ($scope.model.sitetoUnblock) {
                    $scope.model.sitetoIsUnblocked = true;

                    // Update siteList in background.js
                    var _index = HelperService.findIndex($scope.siteList, $scope.model.sitetoUnblock);
                    if (_index != -1) {
                        $scope.siteList.splice(_index, 1);
                        location.hash = '';
                        handleSaving();
                    }
                } else {
                    $('#accessDashboardPassword').modal('toggle');
                }
            } else {
                alert('Wrong password');
                $scope.model.accessPassword = '';
                focus('#access_user_password', 10);
            }
        };

        document.addEventListener('click', function(event) {
            var target = event.target.closest('.notification-li');
            if (!target) {
                $scope.not.show = false;
            }
        });

        function readStorageConfig() {
            $scope.checkbox.subdomainOption = parseInt(HelperService.getKey('subdomainOption'));
            $scope.checkbox.iframeOption = parseInt(HelperService.getKey('iframeOption'));
            $scope.checkbox.whiteList = parseInt(HelperService.getKey('whiteList')) === 1;
            $scope.checkbox.workStatus = parseInt(HelperService.getKey('workStatus'));
            $scope.tagList = JSON.parse(HelperService.getKey('blockKeyList')) || [];
            $scope.days = JSON.parse(HelperService.getKey('days_config')) || [];
        }

        $scope.sortList = function() {
            $scope.model.sortReverse = !$scope.model.sortReverse;
        };

        $scope.addSite = function() {
            $scope.model.newSite = $scope.model.newSite.trim();
            if (!$scope.model.newSite) {
                $('#custom_site').focus();
                return;
            }
            if (HelperService.isUrl($scope.model.newSite)) {
                var site_host = HelperService.getHost($scope.model.newSite);
                if ($scope.siteList.indexOf(site_host) == -1) {
                    addSingleSite();
                } else {
                    alert('Site already exists in blocked list');
                    focus('#custom_site', 100);
                }
            } else {
                $scope.model.errorSite = true;
            }
        };

        $scope.addBlockKey = function() {
            $scope.model.newBlockKey = $scope.model.newBlockKey.trim();
            if (!$scope.model.newBlockKey) {
                $('#add-block-key').focus();
                return;
            }
            if ($scope.tagList.indexOf($scope.model.newBlockKey) == -1) {
                $scope.tagList.push($scope.model.newBlockKey);
                saveKeys($scope.tagList);
            }
            $scope.model.newBlockKey = '';
        };

        $scope.removeBlockKey = function(index) {
            $scope.tagList.splice(index, 1);
            saveKeys($scope.tagList);
        };

        $scope.removeAllKeys = function() {
            if (!confirm('Are you sure ?')) {
                return;
            }
            $scope.tagList = [];
            saveKeys($scope.tagList);
        };

        function saveKeys(keys) {
            localStorage.setItem('blockKeyList', JSON.stringify(keys));
            $.notify('Changes saved !', { globalPosition: 'top center', className: 'success' });
        }

        function addSingleSite() {
            var site = HelperService.getHost($scope.model.newSite);
            $scope.siteList.push(site);
            $scope.model.newSite = '';
            $scope.model.errorSite = false;
            $scope.saveChanges();
        }

        $scope.removeSite = function(site) {
            var _index = HelperService.findIndex($scope.siteList, site);
            if (_index == -1) return;
            $scope.siteList.splice(_index, 1);
            $scope.saveChanges();
        };

        function handleSaving() {
            var sites = JSON.stringify($scope.siteList);
            chrome.runtime.sendMessage(
                {
                    method: 'save_data',
                    value: sites,
                },
                function(response) {
                    if (parseInt(response.status) == 1) {
                        $.notify('Changes saved !', { globalPosition: 'top center', className: 'success' });
                    } else {
                        alert('Error to saving data');
                    }
                }
            );
        }

        $scope.toggleDaySelect = function(index) {
            var idx = $scope.days.indexOf(index);

            if (idx > -1) {
                $scope.days.splice(idx, 1);
            }

            else {
                $scope.days.push(index);
            }
        }

        $scope.showExportModal = function() {
            var data = $scope.siteList.join('\n');
            $scope.model.exportDataModel = data;
        };

        $scope.showLoadModal = function() {
            $scope.model.loadDataListModel = '';
            $scope.model.loadDataResult = [];
        };

        $scope.showHourPage = function() {
            $scope.checkbox.optionHour = parseInt(HelperService.getKey('hour_option'));
            $scope.model.hour_config_extra = HelperService.getExtraHour();
            $scope.model.currentPage = 'whiteTime';
        };

        $scope.showExtraConfigModal = function() {
            readStorageConfig();
            $scope.model.currentPage = 'extra_config';
        };

        $scope.loadSiteDataList = function() {
            $scope.model.loadDataListModel = $scope.model.loadDataListModel.trim();
            $scope.model.loadDataResult = [];

            if ($scope.model.loadDataListModel.length == 0) {
                $('#site_list').focus();
                return;
            }
            $scope.model.loadDataArray = $scope.model.loadDataListModel.split(/\n/);

            $scope.model.loadDataArray.forEach(function(site) {
                site = site.trim();
                site = HelperService.getHost(site);
                var obj = {
                    className: '',
                    site: site,
                };
                if (!HelperService.isUrl(site)) {
                    obj.message = ' is not valid site';
                    obj.className = 'not-valid';
                    $scope.model.loadDataResult.push(obj);
                    return;
                }

                if ($scope.siteList.indexOf(site) == -1) {
                    obj.message = ' added to list';
                    obj.className = 'added';
                    $scope.siteList.push(site);
                } else {
                    obj.className = 'exists';
                    obj.message = ' already exists in blocked list';
                }
                $scope.model.loadDataResult.push(obj);
            });
            $scope.saveChanges();
        };

        $scope.setPassword = function() {
            $('#password').focus();
            if (!$scope.model.userPassword) return;

            if (HelperService.getPass()) {
                var data = prompt('Enter old password');
                if (!data) return;
                if (!HelperService.isPasswordValid(data)) {
                    alert('Wrong password');
                    return;
                }
                handlePasswordSaving();
            } else {
                handlePasswordSaving();
            }
        };

        $scope.removePassword = function() {
            if (!HelperService.getPass()) return;

            var data = prompt('Enter current password');
            if (!data) return;

            if (HelperService.isPasswordValid(data)) {
                removePassword();
                alert('Password removed !');
            } else {
                alert('Wrong Password');
            }
        };

        function removePassword() {
            localStorage.removeItem('pass');
            $scope.model.showPassInput = HelperService.getPass();
        }

        function handlePasswordSaving() {
            localStorage.setItem('pass', HelperService.md5($scope.model.userPassword));
            $scope.model.showPassInput = HelperService.getPass();
            $scope.model.userPassword = '';
            alert('Password saved!');
        }

        $scope.saveHourChanges = function() {
            handleHourConfigSaving();
        };

        function handleHourConfigSaving() {
            if ($scope.checkbox.optionHour) {
                localStorage.setItem('hour_option', 1);
                localStorage.setItem('hour_config_extra', JSON.stringify($scope.model.hour_config_extra));
                localStorage.setItem('days_config', JSON.stringify($scope.days));
            } else {
                localStorage.setItem('hour_option', 0);
            }
            $.notify('Changes Saved', { globalPosition: 'top center', className: 'success' });
        }

        function focus(id, delay) {
            $timeout(function() {
                $(id).focus();
            }, delay);
        }

        $scope.saveWorkStatus = function() {
            setTimeout(function() {
                var workStatus = $scope.checkbox.workStatus ? 1 : 0;
                var text = workStatus ? 'Site Blocker is enabled!' : 'Site Blocker is disabled';
                var className = workStatus ? 'success' : 'error';
                localStorage.setItem('workStatus', workStatus);
                $.notify(text, { globalPosition: 'top center', className: className });
            }, 100);
        };

        $scope.saveExtraConfig = function($event) {
            setTimeout(function() {
                var subdomainOption = $scope.checkbox.subdomainOption ? 1 : 0;
                var iframeOption = $scope.checkbox.iframeOption ? 1 : 0;
                var optionCloseTabs = $scope.checkbox.optionCloseTabs ? 1 : 0;
                var whiteList = $scope.checkbox.whiteList ? 1 : 0;
                var workStatus = $scope.checkbox.workStatus ? 1 : 0;

                localStorage.setItem('blockKeyList', JSON.stringify($scope.tagList));
                localStorage.setItem('subdomainOption', subdomainOption);
                localStorage.setItem('iframeOption', iframeOption);
                localStorage.setItem('close_option', optionCloseTabs);
                localStorage.setItem('whiteList', whiteList);
                localStorage.setItem('workStatus', workStatus);
                $.notify('Changes Saved !', { globalPosition: 'top center', className: 'success' });
            }, 100);
        };

        function loadNotifications() {
            // Todo change url            
            var url = 'https://chess-master.info/site/notifications';
            $http.get(url).then(function(response) {
                if (response.data) {
                    $scope.not.list = response.data || [];
                    localStorage.setItem('notifications', JSON.stringify($scope.not.list));
                    localStorage.setItem('lastUpdate', +new Date());
                    $scope.not = HelperService.getReadMessages($scope.not);
                }
            });
            
        }

        $scope.showSetPasswordModal = function() {
            $scope.model.userPassword = '';
        };

        $scope.clearList = function() {
            if (confirm('Clear blockes list ?')) {
                $scope.siteList = [];
                $scope.saveChanges();
            }
        };

        $scope.saveChanges = function() {
            localStorage.setItem('sites', JSON.stringify(this.siteList));
            handleSaving();
        };

        $scope.saveRedirectUrl = function() {
            if (HelperService.isUrl($scope.model.redirect_url) || !$scope.model.redirect_url) {
                if ($scope.model.redirect_url && !$scope.model.redirect_url.includes('://')) {
                    $scope.model.redirect_url = 'http://' + $scope.model.redirect_url;
                }
                //Update redirect url value in background
                chrome.runtime.sendMessage(
                    {
                        method: 'update_redirect_url',
                        value: $scope.model.redirect_url,
                    },
                    function(response) {
                        if (parseInt(response.status) == 1) {
                            $.notify('Changes saved !', { globalPosition: 'top center', className: 'success' });
                        } else {
                            alert('Error to saving data');
                        }
                    }
                );
            } else {
                alert('Invalid Url');
                focus('#redirect_url', 100);
            }
        };

        $scope.removeRedirectUrl = function() {
            $scope.model.redirect_url = '';
            $scope.saveRedirectUrl();
        };

        $scope.copyText = function myFunction() {
            var copyText = document.getElementById('exportTxt');
            copyText.select();
            document.execCommand('copy');
            $('#copyText').text('Copied !');
            setTimeout(function() {
                $('#copyText').text('Copy');
            }, 800);
        };

        $scope.setRead = function(id) {
            if (!$scope.not.read.includes(id)) {
                $scope.not.read.push(id);
                // Update notifications
                localStorage.setItem('read', JSON.stringify($scope.not.read));
                $scope.not = HelperService.getReadMessages($scope.not);
            }
        };
    }
]);
